import javax.sound.midi.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * The application's toolbar, with the most frequently used commands.
 * 
 * @author  Bjrn Lanneskog
 */
public class MooToolbar extends JToolBar {

	private JButton rewind, playpause, stop, fastforward;
	private JLabel measure, beats, ticks, measuresValue, beatsValue, ticksValue;
	private JPanel progIndPanel;
	private ImageIcon playIcon, pauseIcon;
	private MAdapter mouseAdapter;
	private int ticksPerBeat; 
	public static final int beatsPerMeasure = 4;
	public static final Color bgColor = new Color(192, 224, 255);
	
	/**
	 * Creates the toolbar.
	 */
	
	public MooToolbar() {
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		// setAlignmentX(LEFT_ALIGNMENT);
		setFloatable(false);
		mouseAdapter = new MAdapter();

		// Creates playback buttons
		rewind = createButton("images/rewind.gif", "Rewind");
		playpause = createButton("images/play.gif", "Play");
		stop = createButton("images/stop.gif", "Stop");
		fastforward = createButton("images/forward.gif", "Fast forward");
		playIcon = new ImageIcon("images/play.gif");
		pauseIcon = new ImageIcon("images/pause.gif");

		// Adds playback buttons
		add(rewind);
		add(playpause);
		add(stop);
		add(fastforward);
		
		// Creates progress indicator
		measure = createLabel("Msr", 10);
		beats = createLabel("Beat", 10);
		ticks = createLabel("Tick", 10);
		measuresValue = formatProgInd(createLabel("1", 16));
		beatsValue = formatProgInd(createLabel("1", 16));
		ticksValue = formatProgInd(createLabel("1", 16));
		JPanel spacenorth = new JPanel();
		spacenorth.setBackground(bgColor);
		JPanel spacesouth = new JPanel();
		spacesouth.setBackground(bgColor);

		// Creates progress indicator panel and adds components
		progIndPanel = new JPanel();
		progIndPanel.setMaximumSize(new Dimension(120,27));
		progIndPanel.setLayout(new GridLayout(2,4));
		progIndPanel.add(spacenorth);
		progIndPanel.add(measure);
		progIndPanel.add(beats);
		progIndPanel.add(ticks);
		progIndPanel.add(spacesouth);
		progIndPanel.add(measuresValue);
		progIndPanel.add(beatsValue);
		progIndPanel.add(ticksValue);
		add(progIndPanel);
	}

	/**
	 * Updates the progress indicator.
	 * @param tickPosition	the tick position to visualize
	 */
	public void updateProgInd(long tickPosition) {
		System.out.print("Updating to " + tickPosition + " = ");
		if (tickPosition == 0) {
			resetProgInd();
			System.out.println("1:1:1");
		} else {
			int[] position = Moosique.getPositionForTicks(tickPosition);
			System.out.println("" + position[0] + ":" + position[1] + ":" + position[2]);
			measuresValue.setText(Integer.toString(position[0]));
			beatsValue.setText(Long.toString(position[1]));
			ticksValue.setText(Long.toString(position[2]));
		}
	}

	/**
	 * Resets the progress indicator.
	 */
	public void resetProgInd() {
		measuresValue.setText("1");
		beatsValue.setText("1");
		ticksValue.setText("1");
		updatePlayButton("Play", playIcon);
		ticksPerBeat = Moosique.getSequence().getResolution();
	}

	/**
	 * Creates a button with the specified image and action command / tooltip.
	 */
	private JButton createButton(String imagelocation, String command) {
		JButton button = new JButton (new ImageIcon(imagelocation));
		button.setToolTipText(command);
		button.setActionCommand(command);
		button.addMouseListener(mouseAdapter);
		return button;
	}
	
	/**
	 * Creates labels with the specified text and font size. 
	 */
	private JLabel createLabel(String title, int fontSize){
		JLabel label = new JLabel(title,JLabel.CENTER);
		label.setFont(new Font("Times New Roman", Font.PLAIN, fontSize));
		return label;
	}

	/**
	 * Formats the given label for the progress indicator.
	 */
	private JLabel formatProgInd(JLabel label){
		label.addMouseListener(mouseAdapter);
		label.setBorder(BorderFactory.createLineBorder(Color.black));
		label.setBackground(Color.white);
		return label;
	}

	/**
	 * Updates the play button with the given command and icon.
	 */
	private void updatePlayButton(String command, Icon icon) {
		playpause.setIcon(icon);
		playpause.setActionCommand(command);
		playpause.setToolTipText(command);
	}

	class MAdapter extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			if (e.getSource() instanceof JButton) {
				String command = ((JButton)e.getSource()).getActionCommand();
				if (command == "Play") {
					updatePlayButton("Pause", pauseIcon);
					Moosique.play();
				} else if (command == "Pause") {
					updatePlayButton("Resume", playIcon);
					Moosique.pause();
				} else if (command == "Resume") {
					updatePlayButton("Pause", pauseIcon);
					Moosique.resume();
				} else if (command == "Rewind") {
					System.out.println("Rewound!");
					// Different implementation, perhaps?
					updatePlayButton("Play", playIcon);
					Moosique.setEditPosition(0);
					Moosique.stop();
				} else if (command == "Fast forward") {
				} else if (command == "Stop") {
					updatePlayButton("Play", playIcon);
					Moosique.stop();
				}
			} else if (e.getSource() instanceof JLabel) {
				long position = Moosique.getEditPosition();
				if (e.getSource().equals(measuresValue)) {
					if (SwingUtilities.isRightMouseButton(e)) {
						position += beatsPerMeasure * ticksPerBeat;
					} else if (SwingUtilities.isLeftMouseButton(e) && Integer.parseInt(measuresValue.getText()) > 1) {
						position -= beatsPerMeasure * ticksPerBeat;
					}
				} else if (e.getSource().equals(beatsValue)) {
					if (SwingUtilities.isRightMouseButton(e)) {
						position += ticksPerBeat;
					} else if (SwingUtilities.isLeftMouseButton(e) && Integer.parseInt(beatsValue.getText()) > 1) {
						position -= ticksPerBeat;
					}
				} else if (e.getSource().equals(ticksValue)) {
					if (SwingUtilities.isRightMouseButton(e)) {
						position += 1;
					} else if (SwingUtilities.isLeftMouseButton(e) && Integer.parseInt(ticksValue.getText()) > 1) {
						position -= 1;
					}
				}
				Moosique.setEditPosition(position);
				Moosique.getGUI().update(position);
			}
		}

/*		public void mousePressed(MouseEvent e) {
			if (e.getSource() instanceof JButton) {
				if (((JButton)e.getSource()).getToolTipText() == "Rewind") {
					Moosique.rewind(beatsPerMeasure * ticksPerBeat);
				} else if (((JButton)e.getSource()).getToolTipText() == "Fast forward") {
					Moosique.forward(beatsPerMeasure * ticksPerBeat);
				}
			}
		}

		public void mouseReleased(MouseEvent e) {}
*/	}
}
